@@ -42,13 +42,19 @@ let rec with_afl_logging b dbg =
     let afl_area = V.create_local "shared_mem" in
     let op oper args = Cop (oper, args, dbg) in
     Clet(VP.create afl_area,
-      op (Cload (Word_int, Asttypes.Mutable)) [afl_area_ptr dbg],
-      Clet(VP.create cur_pos, op Cxor [op (Cload (Word_int, Asttypes.Mutable))
+      op (Cload ({memory_chunk=Word_int;
+                  mutability=Asttypes.Mutable;
+                  is_atomic=false})) [afl_area_ptr dbg],
+      Clet(VP.create cur_pos, op Cxor [op (Cload {memory_chunk=Word_int;
+                                                  mutability=Asttypes.Mutable;
+                                                  is_atomic=false})
         [afl_prev_loc dbg]; Cconst_int (cur_location, dbg)],
       Csequence(
         op (Cstore(Byte_unsigned, Assignment))
           [op Cadda [Cvar afl_area; Cvar cur_pos];
-            op Cadda [op (Cload (Byte_unsigned, Asttypes.Mutable))
+            op Cadda [op (Cload {memory_chunk=Byte_unsigned;
+                                 mutability=Asttypes.Mutable;
+                                 is_atomic=false})
                         [op Cadda [Cvar afl_area; Cvar cur_pos]];
                       Cconst_int (1, dbg)]],
         op (Cstore(Word_int, Assignment))
