@@ -38,6 +38,8 @@ let reg ppf r =
       fprintf ppf "[si%i]" s
   | Stack(Outgoing s) ->
       fprintf ppf "[so%i]" s
+  | Stack(Domainstate s) ->
+      fprintf ppf "[ds%i]" s
   end
 
 let regs ppf v =
@@ -120,12 +122,16 @@ let operation op arg ppf res =
       (if alloc then "" else " (noalloc)")
   | Istackoffset n ->
       fprintf ppf "offset stack %i" n
-  | Iload(chunk, addr, Immutable) ->
-      fprintf ppf "%s[%a]"
-       (Printcmm.chunk chunk) (Arch.print_addressing reg addr) arg
-  | Iload(chunk, addr, Mutable) ->
-      fprintf ppf "%s mut[%a]"
-       (Printcmm.chunk chunk) (Arch.print_addressing reg addr) arg
+  | Iload { memory_chunk; addressing_mode; mutability=Immutable; is_atomic } ->
+      fprintf ppf "%s %a[%a]"
+       (Printcmm.chunk memory_chunk)
+       (fun pp a -> if a then fprintf pp "atomic" else ()) is_atomic
+       (Arch.print_addressing reg addressing_mode) arg
+  | Iload { memory_chunk; addressing_mode; mutability=Mutable; is_atomic } ->
+      fprintf ppf "%s %a mut[%a]"
+       (Printcmm.chunk memory_chunk)
+       (fun pp a -> if a then fprintf pp "atomic" else ()) is_atomic
+       (Arch.print_addressing reg addressing_mode) arg
   | Istore(chunk, addr, is_assign) ->
       fprintf ppf "%s[%a] := %a %s"
        (Printcmm.chunk chunk)
@@ -148,6 +154,7 @@ let operation op arg ppf res =
   | Iopaque -> fprintf ppf "opaque %a" reg arg.(0)
   | Ispecific op ->
       Arch.print_specific_operation reg op ppf arg
+  | Idls_get -> fprintf ppf "dls_get"
   | Ipoll { return_label } ->
       fprintf ppf "poll call";
       match return_label with
