@@ -22,8 +22,7 @@ open Lambda
 open Instruct
 open Opcodes
 open Cmo_format
-
-module StringSet = Set.Make(String)
+module String = Misc.Stdlib.String
 
 type error = Not_compatible_32 of (string * string)
 exception Error of error
@@ -39,12 +38,15 @@ let marshal_to_channel_with_possibly_32bit_compat ~filename ~kind outchan obj =
 
 
 let report_error ppf (file, kind) =
-  Format.fprintf ppf "Generated %s %S cannot be used on a 32-bit platform" kind file
+  Format.fprintf ppf "Generated %s %S cannot be used on a 32-bit platform"
+                     kind file
 let () =
   Location.register_error_of_exn
     (function
-      | Error (Not_compatible_32 info) -> Some (Location.error_of_printer_file report_error info)
-      | _ -> None
+      | Error (Not_compatible_32 info) ->
+          Some (Location.error_of_printer_file report_error info)
+      | _ ->
+          None
     )
 
 (* Buffering of bytecode *)
@@ -75,7 +77,6 @@ exception AsInt
 let const_as_int = function
   | Const_base(Const_int i) -> i
   | Const_base(Const_char c) -> Char.code c
-  | Const_pointer i -> i
   | _ -> raise AsInt
 
 let is_immed i = immed_min <= i && i <= immed_max
@@ -162,15 +163,15 @@ and slot_for_c_prim name =
 (* Debugging events *)
 
 let events = ref ([] : debug_event list)
-let debug_dirs = ref StringSet.empty
+let debug_dirs = ref String.Set.empty
 
 let record_event ev =
   let path = ev.ev_loc.Location.loc_start.Lexing.pos_fname in
   let abspath = Location.absolute_path path in
-  debug_dirs := StringSet.add (Filename.dirname abspath) !debug_dirs;
+  debug_dirs := String.Set.add (Filename.dirname abspath) !debug_dirs;
   if Filename.is_relative path then begin
     let cwd = Location.rewrite_absolute_path (Sys.getcwd ()) in
-    debug_dirs := StringSet.add cwd !debug_dirs;
+    debug_dirs := String.Set.add cwd !debug_dirs;
   end;
   ev.ev_pos <- !out_position;
   events := ev :: !events
@@ -181,7 +182,7 @@ let init () =
   out_position := 0;
   label_table := Array.make 16 (Label_undefined []);
   reloc_info := [];
-  debug_dirs := StringSet.empty;
+  debug_dirs := String.Set.empty;
   events := []
 
 (* Emission of one instruction *)
@@ -225,8 +226,8 @@ let emit_instr = function
       let org = !out_position in
       List.iter (out_label_with_orig org) lbls
   | Koffsetclosure ofs ->
-      if ofs = -2 || ofs = 0 || ofs = 2
-      then out (opOFFSETCLOSURE0 + ofs / 2)
+      if ofs = -3 || ofs = 0 || ofs = 3
+      then out (opOFFSETCLOSURE0 + ofs / 3)
       else (out opOFFSETCLOSURE; out_int ofs)
   | Kgetglobal q -> out opGETGLOBAL; slot_for_getglobal q
   | Ksetglobal q -> out opSETGLOBAL; slot_for_setglobal q
@@ -238,10 +239,6 @@ let emit_instr = function
           else (out opCONSTINT; out_int i)
       | Const_base(Const_char c) ->
           out opCONSTINT; out_int (Char.code c)
-      | Const_pointer i ->
-          if i >= 0 && i <= 3
-          then out (opCONST0 + i)
-          else (out opCONSTINT; out_int i)
       | Const_block(t, []) ->
           if t = 0 then out opATOM0 else (out opATOM; out_int t)
       | _ ->
@@ -303,6 +300,10 @@ let emit_instr = function
   | Kgetpubmet tag -> out opGETPUBMET; out_int tag; out_int 0
   | Kgetdynmet -> out opGETDYNMET
   | Kevent ev -> record_event ev
+  | Kperform -> out opPERFORM
+  | Kresume -> out opRESUME
+  | Kresumeterm n -> out opRESUMETERM; out_int n
+  | Kreperformterm n -> out opREPERFORMTERM; out_int n
   | Kstop -> out opSTOP
 
 (* Emission of a list of instructions. Include some peephole optimization. *)
@@ -354,8 +355,8 @@ let rec emit = function
       else (out opPUSHENVACC; out_int n);
       emit c
   | Kpush :: Koffsetclosure ofs :: c ->
-      if ofs = -2 || ofs = 0 || ofs = 2
-      then out(opPUSHOFFSETCLOSURE0 + ofs / 2)
+      if ofs = -3 || ofs = 0 || ofs = 3
+      then out(opPUSHOFFSETCLOSURE0 + ofs / 3)
       else (out opPUSHOFFSETCLOSURE; out_int ofs);
       emit c
   | Kpush :: Kgetglobal id :: Kgetfield n :: c ->
@@ -370,10 +371,6 @@ let rec emit = function
           else (out opPUSHCONSTINT; out_int i)
       | Const_base(Const_char c) ->
           out opPUSHCONSTINT; out_int(Char.code c)
-      | Const_pointer i ->
-          if i >= 0 && i <= 3
-          then out (opPUSHCONST0 + i)
-          else (out opPUSHCONSTINT; out_int i)
       | Const_block(t, []) ->
           if t = 0 then out opPUSHATOM0 else (out opPUSHATOM; out_int t)
       | _ ->
@@ -405,12 +402,12 @@ let to_file outchan unit_name objfile ~required_globals code =
   LongString.output outchan !out_buffer 0 !out_position;
   let (pos_debug, size_debug) =
     if !Clflags.debug then begin
-      debug_dirs := StringSet.add
+      debug_dirs := String.Set.add
         (Filename.dirname (Location.absolute_path objfile))
         !debug_dirs;
       let p = pos_out outchan in
       output_value outchan !events;
-      output_value outchan (StringSet.elements !debug_dirs);
+      output_value outchan (String.Set.elements !debug_dirs);
       (p, pos_out outchan - p)
     end else
       (0, 0) in
@@ -442,13 +439,12 @@ let to_memory init_code fun_code =
   init();
   emit init_code;
   emit fun_code;
-  let code = Meta.static_alloc !out_position in
-  LongString.unsafe_blit_to_bytes !out_buffer 0 code 0 !out_position;
-  let reloc = List.rev !reloc_info
-  and code_size = !out_position in
+  let code = LongString.create !out_position in
+  LongString.blit !out_buffer 0 code 0 !out_position;
+  let reloc = List.rev !reloc_info in
   let events = !events in
   init();
-  (code, code_size, reloc, events)
+  (code, reloc, events)
 
 (* Emission to a file for a packed library *)
 
