@@ -132,19 +132,19 @@ open Printf
 let output_primitive_table outchan =
   let prim = all_primitives() in
   for i = 0 to Array.length prim - 1 do
-    fprintf outchan "extern value %s(void);\n" prim.(i)
+    fprintf outchan "extern value %s();\n" prim.(i)
   done;
-  fprintf outchan "typedef value (*c_primitive)(void);\n";
-  fprintf outchan "c_primitive caml_builtin_cprim[] = {\n";
+  fprintf outchan "typedef value (*primitive)();\n";
+  fprintf outchan "primitive caml_builtin_cprim[] = {\n";
   for i = 0 to Array.length prim - 1 do
     fprintf outchan "  %s,\n" prim.(i)
   done;
-  fprintf outchan "  0 };\n";
+  fprintf outchan "  (primitive) 0 };\n";
   fprintf outchan "const char * caml_names_of_builtin_cprim[] = {\n";
   for i = 0 to Array.length prim - 1 do
     fprintf outchan "  \"%s\",\n" prim.(i)
   done;
-  fprintf outchan "  0 };\n"
+  fprintf outchan "  (char *) 0 };\n"
 
 (* Initialization for batch linking *)
 
@@ -184,8 +184,13 @@ let init () =
     Misc.try_finally
       ~always:(fun () -> remove_file primfile)
       (fun () ->
-         if Sys.command(Printf.sprintf "%s -p > %s"
-                          !Clflags.use_runtime primfile) <> 0
+         let cmd =
+           Filename.quote_command
+             !Clflags.use_runtime
+             ~stdout:primfile
+             ["-p"]
+         in
+         if Sys.command cmd <> 0
          then raise(Error(Wrong_vm !Clflags.use_runtime));
          set_prim_table_from_file primfile
       )
