@@ -1,17 +1,18 @@
-(***********************************************************************)
-(*                                                                     *)
-(*                           Objective Caml                            *)
-(*                                                                     *)
-(*          Jerome Vouillon, projet Cristal, INRIA Rocquencourt        *)
-(*          Objective Caml port by John Malecki and Xavier Leroy       *)
-(*                                                                     *)
-(*  Copyright 1996 Institut National de Recherche en Informatique et   *)
-(*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the Q Public License version 1.0.               *)
-(*                                                                     *)
-(***********************************************************************)
-
-(* $Id$ *)
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*           Jerome Vouillon, projet Cristal, INRIA Rocquencourt          *)
+(*           OCaml port by John Malecki and Xavier Leroy                  *)
+(*                                                                        *)
+(*   Copyright 1996 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
 
 open Input_handling
 open Question
@@ -28,10 +29,7 @@ open Primitives
 
 let line_buffer = Lexing.from_function read_user_input
 
-let rec loop ppf =
-  line_loop ppf line_buffer;
-  if !loaded && (not (yes_or_no "The program is running. Quit anyway")) then
-    loop ppf
+let loop ppf = line_loop ppf line_buffer; stop_user_input ()
 
 let current_duration = ref (-1L)
 
@@ -74,7 +72,8 @@ let rec protect ppf restart loop =
       protect ppf restart (function ppf ->
         let b =
           if !current_duration = -1L then begin
-            let msg = sprintf "Restart from time %Ld and try to get closer of the problem" time in
+            let msg = sprintf "Restart from time %Ld and try to get \
+                               closer of the problem" time in
             stop_user_input ();
             if yes_or_no msg then
               (current_duration := init_duration; true)
@@ -105,8 +104,7 @@ let rec protect ppf restart loop =
             restart ppf
           end)
   | x ->
-      kill_program ();
-      raise x
+      cleanup x kill_program
 
 let execute_file_if_any () =
   let buffer = Buffer.create 128 in
@@ -132,7 +130,8 @@ let execute_file_if_any () =
   let len = Buffer.length buffer in
   if len > 0 then
     let commands = Buffer.sub buffer 0 (pred len) in
-    line_loop Format.std_formatter (Lexing.from_string commands)
+    line_loop Format.std_formatter (Lexing.from_string commands);
+    stop_user_input ()
 
 let toplevel_loop () =
   interactif := false;
@@ -153,38 +152,55 @@ let add_include d =
     Misc.expand_directory Config.standard_library d :: !default_load_path
 let set_socket s =
   socket_name := s
+let set_topdirs_path s =
+  topdirs_path := s
 let set_checkpoints n =
   checkpoint_max_count := n
 let set_directory dir =
   Sys.chdir dir
 let print_version () =
-  printf "The Objective Caml debugger, version %s@." Sys.ocaml_version;
-  exit 0;
-;;
+  printf "The OCaml debugger, version %s@." Sys.ocaml_version;
+  exit 0
+
 let print_version_num () =
   printf "%s@." Sys.ocaml_version;
-  exit 0;
-;;
+  exit 0
 
 let speclist = [
    "-c", Arg.Int set_checkpoints,
       "<count>  Set max number of checkpoints kept";
    "-cd", Arg.String set_directory,
       "<dir>  Change working directory";
-   "-emacs", Arg.Set emacs,
-      "For running the debugger under emacs";
+   "-emacs", Arg.Tuple [Arg.Set emacs; Arg.Set machine_readable],
+      "For running the debugger under emacs; implies -machine-readable";
    "-I", Arg.String add_include,
       "<dir>  Add <dir> to the list of include directories";
+   "-machine-readable", Arg.Set machine_readable,
+      "Print information in a format more suitable for machines";
    "-s", Arg.String set_socket,
       "<filename>  Set the name of the communication socket";
    "-version", Arg.Unit print_version,
       " Print version and exit";
    "-vnum", Arg.Unit print_version_num,
       " Print version number and exit";
+   "-no-version", Arg.Clear Parameters.version,
+      " Do not print version at startup";
+   "-no-prompt", Arg.Clear Parameters.prompt,
+      " Suppress all prompts";
+   "-no-time", Arg.Clear Parameters.time,
+      " Do not print times";
+   "-no-breakpoint-message", Arg.Clear Parameters.breakpoint,
+      " Do not print message at breakpoint setup and removal";
+   "-topdirs-path", Arg.String set_topdirs_path,
+      " Set path to the directory containing topdirs.cmi";
    ]
 
 let function_placeholder () =
-  raise Not_found
+  failwith "custom printer tried to invoke a function from the debuggee"
+
+let report report_error error =
+  eprintf "Debugger [version %s] environment error:@ @[@;%a@]@.;"
+    Config.version report_error error
 
 let main () =
   Callback.register "Debugger.function_placeholder" function_placeholder;
@@ -194,9 +210,9 @@ let main () =
         "Win32" ->
           (Unix.string_of_inet_addr Unix.inet_addr_loopback)^
           ":"^
-          (string_of_int (10000 + ((Unix.getpid ()) mod 10000)))
-      | _ -> Filename.concat Filename.temp_dir_name
-                                ("camldebug" ^ (string_of_int (Unix.getpid ())))
+          (Int.to_string (10000 + ((Unix.getpid ()) mod 10000)))
+      | _ -> Filename.concat (Filename.get_temp_dir_name ())
+                                ("camldebug" ^ (Int.to_string (Unix.getpid ())))
       );
     begin try
       Arg.parse speclist anonymous "";
@@ -210,20 +226,20 @@ let main () =
         arguments := !arguments ^ " " ^ (Filename.quote Sys.argv.(j))
       done
     end;
-    printf "\tObjective Caml Debugger version %s@.@." Config.version;
-    Config.load_path := !default_load_path;
+    if !Parameters.version
+    then printf "\tOCaml Debugger version %s@.@." Config.version;
+    Loadprinter.init();
+    Load_path.init ~auto_include:Compmisc.auto_include !default_load_path;
     Clflags.recursive_types := true;    (* Allow recursive types. *)
     toplevel_loop ();                   (* Toplevel. *)
     kill_program ();
     exit 0
   with
-    Toplevel ->
+  | Toplevel ->
       exit 2
-  | Env.Error e ->
-      eprintf "Debugger [version %s] environment error:@ @[@;" Config.version;
-      Env.report_error err_formatter e;
-      eprintf "@]@.";
+  | Persistent_env.Error e ->
+      report Persistent_env.report_error e;
+      exit 2
+  | Cmi_format.Error e ->
+      report Cmi_format.report_error e;
       exit 2
-
-let _ =
-  Printexc.catch (Unix.handle_unix_error main) ()
