@@ -19,7 +19,7 @@ module String = Misc.Stdlib.String
 let ppf = Format.err_formatter
 (* Print the dependencies *)
 
-type file_kind = ML | MLI;;
+type file_kind = ML | MLI
 
 let load_path = ref ([] : (string * string array) list)
 let ml_synonyms = ref [".ml"]
@@ -194,7 +194,6 @@ let print_filename s =
     loop 0 0;
     print_bytes result;
   end
-;;
 
 let print_dependencies target_files deps =
   let pos = ref 0 in
@@ -559,7 +558,6 @@ let parse_map fname =
   end;
   let mm = Depend.(weaken_map (String.Set.singleton modname) mm) in
   module_map := String.Map.add modname mm !module_map
-;;
 
 (* Dependency processing *)
 
@@ -578,94 +576,100 @@ let process_dep_args dep_args = List.iter process_dep_arg dep_args
 
 let print_version () =
   Format.printf "ocamldep, version %s@." Sys.ocaml_version;
-  exit 0;
-;;
+  exit 0
 
 let print_version_num () =
   Format.printf "%s@." Sys.ocaml_version;
-  exit 0;
-;;
+  exit 0
 
 
 let run_main argv =
   let dep_args_rev : dep_arg list ref = ref [] in
   let add_dep_arg f s = dep_args_rev := (f s) :: !dep_args_rev in
   Clflags.classic := false;
-  Compenv.readenv ppf Before_args;
-  Clflags.reset_arguments (); (* reset arguments from ocamlc/ocamlopt *)
-  Clflags.add_arguments __LOC__ [
-     "-absname", Arg.Set Clflags.absname,
+  try
+    Compenv.readenv ppf Before_args;
+    Clflags.reset_arguments (); (* reset arguments from ocamlc/ocamlopt *)
+    Clflags.add_arguments __LOC__ [
+      "-absname", Arg.Set Clflags.absname,
         " Show absolute filenames in error messages";
-     "-all", Arg.Set all_dependencies,
+      "-all", Arg.Set all_dependencies,
         " Generate dependencies on all files";
-     "-allow-approx", Arg.Set allow_approximation,
+      "-allow-approx", Arg.Set allow_approximation,
         " Fallback to a lexer-based approximation on unparsable files";
-     "-as-map", Arg.Set Clflags.transparent_modules,
-      " Omit delayed dependencies for module aliases (-no-alias-deps -w -49)";
-      (* "compiler uses -no-alias-deps, and no module is coerced"; *)
-     "-debug-map", Arg.Set debug,
+      "-as-map", Arg.Set Clflags.transparent_modules,
+        " Omit delayed dependencies for module aliases (-no-alias-deps -w -49)";
+        (* "compiler uses -no-alias-deps, and no module is coerced"; *)
+      "-debug-map", Arg.Set debug,
         " Dump the delayed dependency map for each map file";
-     "-I", Arg.String (add_to_list Clflags.include_dirs),
+      "-I", Arg.String (add_to_list Clflags.include_dirs),
         "<dir>  Add <dir> to the list of include directories";
-     "-nocwd", Arg.Set nocwd,
+      "-nocwd", Arg.Set nocwd,
         " Do not add current working directory to \
           the list of include directories";
-     "-impl", Arg.String (add_dep_arg (fun f -> Src (f, Some ML))),
+      "-impl", Arg.String (add_dep_arg (fun f -> Src (f, Some ML))),
         "<f>  Process <f> as a .ml file";
-     "-intf", Arg.String (add_dep_arg (fun f -> Src (f, Some MLI))),
+      "-intf", Arg.String (add_dep_arg (fun f -> Src (f, Some MLI))),
         "<f>  Process <f> as a .mli file";
-     "-map", Arg.String (add_dep_arg (fun f -> Map f)),
+      "-map", Arg.String (add_dep_arg (fun f -> Map f)),
         "<f>  Read <f> and propagate delayed dependencies to following files";
-     "-ml-synonym", Arg.String(add_to_synonym_list ml_synonyms),
+      "-ml-synonym", Arg.String(add_to_synonym_list ml_synonyms),
         "<e>  Consider <e> as a synonym of the .ml extension";
-     "-mli-synonym", Arg.String(add_to_synonym_list mli_synonyms),
+      "-mli-synonym", Arg.String(add_to_synonym_list mli_synonyms),
         "<e>  Consider <e> as a synonym of the .mli extension";
-     "-modules", Arg.Set raw_dependencies,
+      "-modules", Arg.Set raw_dependencies,
         " Print module dependencies in raw form (not suitable for make)";
-     "-native", Arg.Set native_only,
+      "-native", Arg.Set native_only,
         " Generate dependencies for native-code only (no .cmo files)";
-     "-bytecode", Arg.Set bytecode_only,
+      "-bytecode", Arg.Set bytecode_only,
         " Generate dependencies for bytecode-code only (no .cmx files)";
-     "-one-line", Arg.Set one_line,
+      "-one-line", Arg.Set one_line,
         " Output one line per file, regardless of the length";
-     "-open", Arg.String (add_to_list Clflags.open_modules),
+      "-open", Arg.String (add_to_list Clflags.open_modules),
         "<module>  Opens the module <module> before typing";
-     "-plugin", Arg.String(fun _p -> Clflags.plugin := true),
-         "<plugin>  (no longer supported)";
-     "-pp", Arg.String(fun s -> Clflags.preprocessor := Some s),
-         "<cmd>  Pipe sources through preprocessor <cmd>";
-     "-ppx", Arg.String (add_to_list Compenv.first_ppx),
-         "<cmd>  Pipe abstract syntax trees through preprocessor <cmd>";
-     "-shared", Arg.Set shared,
-         " Generate dependencies for native plugin files (.cmxs targets)";
-     "-slash", Arg.Set Clflags.force_slash,
-         " (Windows) Use forward slash / instead of backslash \\ in file paths";
-     "-sort", Arg.Set sort_files,
+      "-plugin", Arg.String(fun _p -> Clflags.plugin := true),
+        "<plugin>  (no longer supported)";
+      "-pp", Arg.String(fun s -> Clflags.preprocessor := Some s),
+        "<cmd>  Pipe sources through preprocessor <cmd>";
+      "-ppx", Arg.String (add_to_list Compenv.first_ppx),
+        "<cmd>  Pipe abstract syntax trees through preprocessor <cmd>";
+      "-shared", Arg.Set shared,
+        " Generate dependencies for native plugin files (.cmxs targets)";
+      "-slash", Arg.Set Clflags.force_slash,
+        " (Windows) Use forward slash / instead of backslash \\ in file paths";
+      "-no-slash", Arg.Clear Clflags.force_slash,
+        " (Windows) Preserve any backslash \\ in file paths";
+      "-sort", Arg.Set sort_files,
         " Sort files according to their dependencies";
-     "-version", Arg.Unit print_version,
-         " Print version and exit";
-     "-vnum", Arg.Unit print_version_num,
-         " Print version number and exit";
-     "-args", Arg.Expand Arg.read_arg,
-         "<file> Read additional newline separated command line arguments \n\
-         \      from <file>";
-     "-args0", Arg.Expand Arg.read_arg0,
-         "<file> Read additional NUL separated command line arguments from \n\
-         \      <file>"
-  ];
-  let usage =
-    Printf.sprintf "Usage: %s [options] <source files>\nOptions are:"
-                   (Filename.basename Sys.argv.(0))
-  in
-  Clflags.parse_arguments argv (add_dep_arg (fun f -> Src (f, None))) usage;
-  process_dep_args (List.rev !dep_args_rev);
-  Compenv.readenv ppf Before_link;
-  if !sort_files then sort_files_by_dependencies !files
-  else List.iter print_file_dependencies (List.sort compare !files);
-  exit (if Error_occurred.get () then 2 else 0)
+      "-version", Arg.Unit print_version,
+        " Print version and exit";
+      "-vnum", Arg.Unit print_version_num,
+        " Print version number and exit";
+      "-args", Arg.Expand Arg.read_arg,
+        "<file> Read additional newline separated command line arguments \n\
+        \      from <file>";
+      "-args0", Arg.Expand Arg.read_arg0,
+        "<file> Read additional NUL separated command line arguments from \n\
+        \      <file>"
+    ];
+    let program = Filename.basename Sys.argv.(0) in
+    Compenv.parse_arguments (ref argv)
+      (add_dep_arg (fun f -> Src (f, None))) program;
+    process_dep_args (List.rev !dep_args_rev);
+    Compenv.readenv ppf Before_link;
+    if !sort_files then sort_files_by_dependencies !files
+    else List.iter print_file_dependencies (List.sort compare !files);
+    (if Error_occurred.get () then 2 else 0)
+  with
+  | Compenv.Exit_with_status n ->
+      n
+  | exn ->
+      Location.report_exception ppf exn;
+      2
+
 
 let main () =
-  run_main Sys.argv
+  exit (run_main Sys.argv)
 
 let main_from_option () =
   if Sys.argv.(1) <> "-depend" then begin
@@ -677,4 +681,4 @@ let main_from_option () =
     Array.concat [ [| Sys.argv.(0) ^ " -depend" |];
                    Array.sub Sys.argv 2 (Array.length Sys.argv - 2) ] in
   Sys.argv.(0) <- args.(0);
-  run_main args
+  exit (run_main args)
