@@ -1,15 +1,17 @@
-(***********************************************************************)
-(*                             OCamldoc                                *)
-(*                                                                     *)
-(*            Maxence Guesdon, projet Cristal, INRIA Rocquencourt      *)
-(*                                                                     *)
-(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
-(*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the Q Public License version 1.0.               *)
-(*                                                                     *)
-(***********************************************************************)
-
-(* $Id: odoc_to_text.ml 8927 2008-07-23 08:55:36Z guesdon $ *)
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*             Maxence Guesdon, projet Cristal, INRIA Rocquencourt        *)
+(*                                                                        *)
+(*   Copyright 2001 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
 
 (** Text generation.
 
@@ -22,7 +24,6 @@ open Type
 open Value
 open Module
 open Class
-open Parameter
 
 (** A class used to get a [text] for info structures. *)
 class virtual info =
@@ -32,7 +33,7 @@ class virtual info =
        Add a pair here to handle a tag.*)
     val mutable tag_functions = ([] : (string * (Odoc_info.text -> Odoc_info.text)) list)
 
-    (** @return [etxt] value for an authors list. *)
+    (** @return [text] value for an authors list. *)
     method text_of_author_list l =
       match l with
         [] ->
@@ -61,6 +62,17 @@ class virtual info =
                     Newline
                   ]
 
+    (** @return [text] value to represent the list of "before" information. *)
+    method text_of_before = function
+      [] -> []
+    | l ->
+        let f (v, text) =
+          (Bold [Raw (Printf.sprintf "%s %s " Odoc_messages.before v) ]) ::
+            text @
+            [Newline]
+        in
+        List.flatten (List.map f l)
+
     (** @return [text] value for the given list of raised exceptions.*)
     method text_of_raised_exceptions l =
       match l with
@@ -86,13 +98,10 @@ class virtual info =
 
     (** Return [text] value for the given "see also" reference. *)
     method text_of_see (see_ref, t)  =
-      let t_ref =
-        match see_ref with
-          Odoc_info.See_url s -> [ Odoc_info.Link (s, t) ]
-        | Odoc_info.See_file s -> (Odoc_info.Code s) :: (Odoc_info.Raw " ") :: t
-        | Odoc_info.See_doc s -> (Odoc_info.Italic [Odoc_info.Raw s]) :: (Odoc_info.Raw " ") :: t
-      in
-      t_ref
+      match see_ref with
+        Odoc_info.See_url s -> [ Odoc_info.Link (s, t) ]
+      | Odoc_info.See_file s -> (Odoc_info.Code s) :: (Odoc_info.Raw " ") :: t
+      | Odoc_info.See_doc s -> (Odoc_info.Italic [Odoc_info.Raw s]) :: (Odoc_info.Raw " ") :: t
 
     (** Return [text] value for the given list of "see also" references.*)
     method text_of_sees l =
@@ -135,6 +144,18 @@ class virtual info =
         []
         l
 
+    method text_of_alerts alerts =
+      List.concat_map (fun al ->
+          let payload =
+            match al.alert_payload with
+            | Some p -> [ Raw ". "; Raw p ]
+            | None -> []
+          in
+          [ Bold [ Raw Odoc_messages.alert; Raw " "; Raw al.alert_name ] ]
+          @ payload
+          @ [ Newline ]
+        ) alerts
+
     (** @return [text] value for a description, except for the i_params field. *)
     method text_of_info ?(block=true) info_opt =
       match info_opt with
@@ -144,7 +165,7 @@ class virtual info =
           let t =
             (match info.i_deprecated with
               None -> []
-            | Some t -> ( Italic [Raw (Odoc_messages.deprecated^" ")] ) :: t
+            | Some t -> ( Italic [Raw (Odoc_messages.deprecated^". ")] ) :: t
              ) @
             (match info.i_desc with
               None -> []
@@ -153,10 +174,12 @@ class virtual info =
             ) @
             (self#text_of_author_list info.i_authors) @
             (self#text_of_version_opt info.i_version) @
+            (self#text_of_before info.i_before) @
             (self#text_of_since_opt info.i_since) @
             (self#text_of_raised_exceptions info.i_raised_exceptions) @
             (self#text_of_return_opt info.i_return_value) @
             (self#text_of_sees info.i_sees) @
+            (self#text_of_alerts info.i_alerts) @
             (self#text_of_custom info.i_custom)
           in
           if block then
@@ -181,12 +204,10 @@ class virtual to_text =
         let rel = Name.get_relative m_name match_s in
         Odoc_info.apply_if_equal Odoc_info.use_hidden_modules match_s rel
       in
-      let s2 = Str.global_substitute
-          (Str.regexp "\\([A-Z]\\([a-zA-Z_'0-9]\\)*\\.\\)+\\([a-z][a-zA-Z_'0-9]*\\)")
-          f
-          s
-      in
-      s2
+      Str.global_substitute
+        (Str.regexp "\\([A-Z]\\([a-zA-Z_'0-9]\\)*\\.\\)+\\([a-z][a-zA-Z_'0-9]*\\)")
+        f
+        s
 
     (** Take a string and return the string where fully qualified idents
        have been replaced by idents relative to the given module name.
@@ -197,12 +218,10 @@ class virtual to_text =
         let rel = Name.get_relative m_name match_s in
         Odoc_info.apply_if_equal Odoc_info.use_hidden_modules match_s rel
       in
-      let s2 = Str.global_substitute
-          (Str.regexp "\\([A-Z]\\([a-zA-Z_'0-9]\\)*\\.\\)+\\([A-Z][a-zA-Z_'0-9]*\\)")
-          f
-          s
-      in
-      s2
+      Str.global_substitute
+        (Str.regexp "\\([A-Z]\\([a-zA-Z_'0-9]\\)*\\.\\)+\\([A-Z][a-zA-Z_'0-9]*\\)")
+        f
+        s
 
     (** Get a string for a [Types.class_type] where all idents are relative. *)
     method normal_class_type m_name t =
@@ -220,6 +239,11 @@ class virtual to_text =
     method normal_type_list ?par m_name sep t =
       self#relative_idents m_name (Odoc_info.string_of_type_list ?par sep t)
 
+    method normal_cstr_args ?par m_name = function
+      | Cstr_tuple l -> self#normal_type_list ?par m_name " * " l
+      | Cstr_record r -> self#relative_idents m_name
+                            (Odoc_str.string_of_record r)
+
     (** Get a string for a list of class or class type type parameters
        where all idents are relative. *)
     method normal_class_type_param_list m_name t =
@@ -229,18 +253,16 @@ class virtual to_text =
     method normal_class_params m_name c =
       let s = Odoc_info.string_of_class_params c in
       self#relative_idents m_name
-	(Odoc_info.remove_ending_newline s)
+        (Odoc_info.remove_ending_newline s)
 
     (** @return [text] value to represent a [Types.type_expr].*)
     method text_of_type_expr module_name t =
-      let t = List.flatten
-          (List.map
-             (fun s -> [Code s ; Newline ])
-             (Str.split (Str.regexp "\n")
-                (self#normal_type module_name t))
-          )
-      in
-      t
+      List.flatten
+        (List.map
+           (fun s -> [Code s ; Newline ])
+           (Str.split (Str.regexp "\n")
+              (self#normal_type module_name t))
+        )
 
     (** Return [text] value for a given short [Types.type_expr].*)
     method text_of_short_type_expr module_name t =
@@ -256,17 +278,15 @@ class virtual to_text =
     method text_of_class_type_param_expr_list module_name l =
       [ Code (self#normal_class_type_param_list module_name l) ]
 
-    (** @return [text] value to represent parameters of a class (with arraows).*)
+    (** @return [text] value to represent parameters of a class (with arrows).*)
     method text_of_class_params module_name c =
-      let t = Odoc_info.text_concat
-	  [Newline]
-          (List.map
-             (fun s -> [Code s])
-	     (Str.split (Str.regexp "\n")
-                (self#normal_class_params module_name c))
-	  )
-      in
-      t
+      Odoc_info.text_concat
+        [Newline]
+        (List.map
+           (fun s -> [Code s])
+           (Str.split (Str.regexp "\n")
+              (self#normal_class_params module_name c))
+        )
 
     (** @return [text] value to represent a [Types.module_type]. *)
     method text_of_module_type t =
@@ -280,10 +300,10 @@ class virtual to_text =
       let name = v.val_name in
       let s_name = Name.simple name in
       let s =
-	Format.fprintf Format.str_formatter "@[<hov 2>val %s :@ %s"
+        Format.fprintf Format.str_formatter "@[<hov 2>val %s :@ %s"
           s_name
           (self#normal_type (Name.father v.val_name) v.val_type);
-	Format.flush_str_formatter ()
+        Format.flush_str_formatter ()
       in
       [ CodePre s ] @
       [Latex ("\\index{"^(self#label s_name)^"@\\verb`"^(self#label ~no_:false s_name)^"`}\n")] @
@@ -294,12 +314,12 @@ class virtual to_text =
       let s_name = Name.simple a.att_value.val_name in
       let mod_name = Name.father a.att_value.val_name in
       let s =
-	Format.fprintf Format.str_formatter "@[<hov 2>val %s%s%s :@ %s"
+        Format.fprintf Format.str_formatter "@[<hov 2>val %s%s%s :@ %s"
           (if a.att_virtual then "virtual " else "")
           (if a.att_mutable then "mutable " else "")
           s_name
-	  (self#normal_type mod_name a.att_value.val_type);
-	Format.flush_str_formatter ()
+          (self#normal_type mod_name a.att_value.val_type);
+        Format.flush_str_formatter ()
       in
       (CodePre s) ::
       [Latex ("\\index{"^(self#label s_name)^"@\\verb`"^(self#label ~no_:false s_name)^"`}\n")] @
@@ -310,12 +330,12 @@ class virtual to_text =
       let s_name = Name.simple m.met_value.val_name in
       let mod_name = Name.father m.met_value.val_name in
       let s =
-	Format.fprintf Format.str_formatter "@[<hov 2>method %s%s%s :@ %s"
+        Format.fprintf Format.str_formatter "@[<hov 2>method %s%s%s :@ %s"
           (if m.met_private then "private " else "")
           (if m.met_virtual then "virtual " else "")
           s_name
-	  (self#normal_type mod_name m.met_value.val_type);
-	Format.flush_str_formatter ()
+          (self#normal_type mod_name m.met_value.val_type);
+        Format.flush_str_formatter ()
       in
       (CodePre s) ::
       [Latex ("\\index{"^(self#label s_name)^"@\\verb`"^(self#label ~no_:false s_name)^"`}\n")] @
@@ -325,29 +345,36 @@ class virtual to_text =
     (** @return [text] value for an exception. *)
     method text_of_exception e =
       let s_name = Name.simple e.ex_name in
+      let father = Name.father e.ex_name in
       Format.fprintf Format.str_formatter "@[<hov 2>exception %s" s_name ;
-        (match e.ex_args with
-          [] -> ()
-        | _ ->
-            Format.fprintf Format.str_formatter "@ of "
-        );
-      let s = self#normal_type_list
-	  ~par: false (Name.father e.ex_name) " * " e.ex_args
-      in
-      let s2 =
-        Format.fprintf Format.str_formatter "%s" s ;
-        (match e.ex_alias with
-          None -> ()
-        | Some ea ->
-            Format.fprintf Format.str_formatter " = %s"
-              (
-               match ea.ea_ex with
-                 None -> ea.ea_name
-               | Some e -> e.ex_name
-              )
-        );
-        Format.flush_str_formatter ()
-      in
+      (match e.ex_args, e.ex_ret with
+         Cstr_tuple [], None -> ()
+       | Cstr_tuple [], Some r ->
+           Format.fprintf Format.str_formatter " %s@ %s"
+             ":"
+             (self#normal_type father r)
+       | args, None ->
+           Format.fprintf Format.str_formatter " %s@ %s"
+             "of"
+             (self#normal_cstr_args ~par:false father args)
+       | args, Some r ->
+           Format.fprintf Format.str_formatter " %s@ %s@ %s@ %s"
+             ":"
+             (self#normal_cstr_args ~par:false father args)
+             "->"
+             (self#normal_type father r)
+      );
+      (match e.ex_alias with
+         None -> ()
+       | Some ea ->
+           Format.fprintf Format.str_formatter " = %s"
+            (
+              match ea.ea_ex with
+                None -> ea.ea_name
+              | Some e -> e.ex_name
+            )
+      );
+      let s2 = Format.flush_str_formatter () in
       [ CodePre s2 ] @
       [Latex ("\\index{"^(self#label s_name)^"@\\verb`"^(self#label ~no_:false s_name)^"`}\n")] @
       (self#text_of_info e.ex_info)
@@ -417,8 +444,11 @@ class virtual to_text =
             List
               (List.map
                  (fun (p, desc_opt) ->
-                   [Code (p.mp_name^" : ")] @
-                   (self#text_of_module_type p.mp_type) @
+                   begin match p.mp_type with None -> [Raw ""]
+                   | Some mty ->
+                       [Code (p.mp_name^" : ")] @
+                       (self#text_of_module_type mty)
+                   end @
                    (match desc_opt with
                      None -> []
                    | Some t -> (Raw " ") :: t)
@@ -537,12 +567,25 @@ class virtual to_text =
           [Code ((if with_def_syntax then " : " else "")^
                  Odoc_messages.struct_end^" ")]
 
-      | Module_functor (p, k)  ->
+      | Module_functor (_, k)  ->
           (if with_def_syntax then [Code " : "] else []) @
           [Code "functor ... "] @
           [Code " -> "] @
           (self#text_of_module_kind ~with_def_syntax: false k)
 
+      | Module_typeof s ->
+          let code = Printf.sprintf "%smodule type of %s"
+            (if with_def_syntax then " : " else "")
+            s
+          in
+          [Code code]
+      | Module_unpack (code, _) ->
+          let code = Printf.sprintf "%s%s"
+            (if with_def_syntax then " : " else "")
+            code
+          in
+          [Code code]
+
     (** Return html code for a [module_type_kind].*)
     method text_of_module_type_kind ?(with_def_syntax=true) tk =
       match tk with
@@ -551,7 +594,7 @@ class virtual to_text =
 
       | Module_type_functor (p, k) ->
           let t1 =
-	    [Code ("("^p.mp_name^" : ")] @
+            [Code ("("^p.mp_name^" : ")] @
             (self#text_of_module_type_kind p.mp_kind) @
             [Code ") -> "]
           in
@@ -570,5 +613,9 @@ class virtual to_text =
                  | Some mt -> mt.mt_name))
           ]
 
-
+      | Odoc_module.Module_type_typeof s ->
+          let code = Printf.sprintf "%smodule type of %s"
+            (if with_def_syntax then " = " else "") s
+          in
+          [ Code code ]
   end
