@@ -22,10 +22,10 @@ val afl_instrument : bool
 (** Whether AFL support has been enabled in the compiler *)
 
 val asm : string
-(** Path to the assembler*)
+(** Path to the assembler *)
 
 val cc : string
-(** Path to the C compiler*)
+(** Path to the C compiler *)
 
 val cflags : string
 (** Flags to pass to the C compiler *)
@@ -33,6 +33,12 @@ val cflags : string
 val ccomptype : string
 (** Type of C compiler (msvc, cc, etc.) *)
 
+val diff : string
+(** Path to the diff tool *)
+
+val diff_flags : string
+(** Flags to pass to the diff tool *)
+
 val shared_libraries : bool
 (** [true] if shared libraries are supported, [false] otherwise *)
 
@@ -73,9 +79,6 @@ val ocamlsrcdir : string
 val flambda : bool
 (** Whether flambda has been enabled at configure time *)
 
-val safe_string : bool
-(** Whether the compiler was configured with -safe-string *)
-
 val flat_float_array : bool
 (* Whether the compiler was configured with -flat-float-array *)
 
@@ -121,6 +124,3 @@ val function_sections : bool
 
 val instrumented_runtime : bool
 (** Whether the instrumented runtime is available *)
-
-val naked_pointers : bool
-(** Whether the runtime system supports naked pointers outside the heap *)
