@@ -57,6 +57,8 @@ let find_test_dirs = ref []
 
 let list_tests = ref []
 
+let show_timings = ref false
+
 let add_to_list r x =
   r := !r @ [x]
 
@@ -68,6 +70,8 @@ let commandline_options =
   ("-show-actions", Arg.Unit show_actions, " Show available actions.");
   ("-show-tests", Arg.Unit show_tests, " Show available tests.");
   ("-show-variables", Arg.Unit show_variables, " Show available variables.");
+  ("-show-timings", Arg.Set show_timings,
+   " Show the wall clock time taken for each test file.");
   ("-timeout",
      Arg.Int (fun t -> if t >= 0
                        then default_timeout := t
@@ -95,3 +99,4 @@ let default_timeout = !default_timeout
 let find_test_dirs = !find_test_dirs
 let list_tests = !list_tests
 let keep_test_dir_on_success = !keep_test_dir_on_success
+let show_timings = !show_timings
