@@ -1,21 +1,21 @@
-/***********************************************************************/
-/*                                                                     */
-/*                           Objective Caml                            */
-/*                                                                     */
-/*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         */
-/*                                                                     */
-/*  Copyright 1996 Institut National de Recherche en Informatique et   */
-/*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License, with    */
-/*  the special exception on linking described in file ../../LICENSE.  */
-/*                                                                     */
-/***********************************************************************/
+/**************************************************************************/
+/*                                                                        */
+/*                                 OCaml                                  */
+/*                                                                        */
+/*             Xavier Leroy, projet Cristal, INRIA Rocquencourt           */
+/*                                                                        */
+/*   Copyright 1996 Institut National de Recherche en Informatique et     */
+/*     en Automatique.                                                    */
+/*                                                                        */
+/*   All rights reserved.  This file is distributed under the terms of    */
+/*   the GNU Lesser General Public License version 2.1, with the          */
+/*   special exception on linking described in the file LICENSE.          */
+/*                                                                        */
+/**************************************************************************/
 
-/* $Id$ */
-
-#include <mlvalues.h>
-#include <alloc.h>
-#include <fail.h>
+#include <caml/mlvalues.h>
+#include <caml/alloc.h>
+#include <caml/fail.h>
 #include "unixsupport.h"
 
 #ifdef HAS_TERMIOS
@@ -92,17 +92,22 @@ static long terminal_io_descr[] = {
 #undef cflags
 #undef lflags
 
-struct speedtable_entry ;
-
 static struct {
   speed_t speed;
   int baud;
 } speedtable[] = {
+
+  /* standard speeds */
+  {B0,       0},
   {B50,      50},
   {B75,      75},
   {B110,     110},
   {B134,     134},
   {B150,     150},
+#ifdef B200
+  /* Shouldn't need to be ifdef'd but I'm not sure it's available everywhere. */
+  {B200,     200},
+#endif
   {B300,     300},
   {B600,     600},
   {B1200,    1200},
@@ -112,6 +117,8 @@ static struct {
   {B9600,    9600},
   {B19200,   19200},
   {B38400,   38400},
+
+  /* usual extensions */
 #ifdef B57600
   {B57600,   57600},
 #endif
@@ -121,12 +128,71 @@ static struct {
 #ifdef B230400
   {B230400,  230400},
 #endif
-  {B0,       0}
+
+  /* Linux extensions */
+#ifdef B460800
+  {B460800,  460800},
+#endif
+#ifdef B500000
+  {B500000,  500000},
+#endif
+#ifdef B576000
+  {B576000,  576000},
+#endif
+#ifdef B921600
+  {B921600,  921600},
+#endif
+#ifdef B1000000
+  {B1000000, 1000000},
+#endif
+#ifdef B1152000
+  {B1152000, 1152000},
+#endif
+#ifdef B1500000
+  {B1500000, 1500000},
+#endif
+#ifdef B2000000
+  {B2000000, 2000000},
+#endif
+#ifdef B2500000
+  {B2500000, 2500000},
+#endif
+#ifdef B3000000
+  {B3000000, 3000000},
+#endif
+#ifdef B3500000
+  {B3500000, 3500000},
+#endif
+#ifdef B4000000
+  {B4000000, 4000000},
+#endif
+
+  /* MacOS extensions */
+#ifdef B7200
+  {B7200,    7200},
+#endif
+#ifdef B14400
+  {B14400,   14400},
+#endif
+#ifdef B28800
+  {B28800,   28800},
+#endif
+#ifdef B76800
+  {B76800,   76800},
+#endif
+
+  /* Cygwin extensions (in addition to the Linux ones) */
+#ifdef B128000
+  {B128000,  128000},
+#endif
+#ifdef B256000
+  {B256000,  256000},
+#endif
 };
 
 #define NSPEEDS (sizeof(speedtable) / sizeof(speedtable[0]))
 
-static void encode_terminal_status(value *dst)
+static void encode_terminal_status(volatile value *dst)
 {
   long * pc;
   int i;
@@ -176,7 +242,7 @@ static void encode_terminal_status(value *dst)
   }
 }
 
-static void decode_terminal_status(value *src)
+static void decode_terminal_status(volatile value *src)
 {
   long * pc;
   int i;
@@ -200,7 +266,7 @@ static void decode_terminal_status(value *src)
         if (i >= 0 && i < num) {
           *dst = (*dst & ~msk) | pc[i];
         } else {
-          unix_error(EINVAL, "tcsetattr", Nothing);
+          caml_unix_error(EINVAL, "tcsetattr", Nothing);
         }
         pc += num;
         break; }
@@ -216,11 +282,11 @@ static void decode_terminal_status(value *src)
             case Input:
               res = cfsetispeed(&terminal_status, speedtable[i].speed); break;
             }
-            if (res == -1) uerror("tcsetattr", Nothing);
+            if (res == -1) caml_uerror("tcsetattr", Nothing);
             goto ok;
           }
         }
-        unix_error(EINVAL, "tcsetattr", Nothing);
+        caml_unix_error(EINVAL, "tcsetattr", Nothing);
       ok:
         break; }
     case Char:
@@ -231,13 +297,13 @@ static void decode_terminal_status(value *src)
   }
 }
 
-CAMLprim value unix_tcgetattr(value fd)
+CAMLprim value caml_unix_tcgetattr(value fd)
 {
   value res;
 
   if (tcgetattr(Int_val(fd), &terminal_status) == -1)
-    uerror("tcgetattr", Nothing);
-  res = alloc_tuple(NFIELDS);
+    caml_uerror("tcgetattr", Nothing);
+  res = caml_alloc_tuple(NFIELDS);
   encode_terminal_status(&Field(res, 0));
   return res;
 }
@@ -246,39 +312,44 @@ static int when_flag_table[] = {
   TCSANOW, TCSADRAIN, TCSAFLUSH
 };
 
-CAMLprim value unix_tcsetattr(value fd, value when, value arg)
+CAMLprim value caml_unix_tcsetattr(value fd, value when, value arg)
 {
   if (tcgetattr(Int_val(fd), &terminal_status) == -1)
-    uerror("tcsetattr", Nothing);
+    caml_uerror("tcsetattr", Nothing);
   decode_terminal_status(&Field(arg, 0));
   if (tcsetattr(Int_val(fd),
                 when_flag_table[Int_val(when)],
                 &terminal_status) == -1)
-    uerror("tcsetattr", Nothing);
+    caml_uerror("tcsetattr", Nothing);
   return Val_unit;
 }
 
-CAMLprim value unix_tcsendbreak(value fd, value delay)
+CAMLprim value caml_unix_tcsendbreak(value fd, value delay)
 {
   if (tcsendbreak(Int_val(fd), Int_val(delay)) == -1)
-    uerror("tcsendbreak", Nothing);
+    caml_uerror("tcsendbreak", Nothing);
   return Val_unit;
 }
 
-CAMLprim value unix_tcdrain(value fd)
+#if defined(__ANDROID__)
+CAMLprim value caml_unix_tcdrain(value fd)
+{ caml_invalid_argument("tcdrain not implemented"); }
+#else
+CAMLprim value caml_unix_tcdrain(value fd)
 {
-  if (tcdrain(Int_val(fd)) == -1) uerror("tcdrain", Nothing);
+  if (tcdrain(Int_val(fd)) == -1) caml_uerror("tcdrain", Nothing);
   return Val_unit;
 }
+#endif
 
 static int queue_flag_table[] = {
   TCIFLUSH, TCOFLUSH, TCIOFLUSH
 };
 
-CAMLprim value unix_tcflush(value fd, value queue)
+CAMLprim value caml_unix_tcflush(value fd, value queue)
 {
   if (tcflush(Int_val(fd), queue_flag_table[Int_val(queue)]) == -1)
-    uerror("tcflush", Nothing);
+    caml_uerror("tcflush", Nothing);
   return Val_unit;
 }
 
@@ -286,31 +357,31 @@ static int action_flag_table[] = {
   TCOOFF, TCOON, TCIOFF, TCION
 };
 
-CAMLprim value unix_tcflow(value fd, value action)
+CAMLprim value caml_unix_tcflow(value fd, value action)
 {
   if (tcflow(Int_val(fd), action_flag_table[Int_val(action)]) == -1)
-    uerror("tcflow", Nothing);
+    caml_uerror("tcflow", Nothing);
   return Val_unit;
 }
 
 #else
 
-CAMLprim value unix_tcgetattr(value fd)
-{ invalid_argument("tcgetattr not implemented"); }
+CAMLprim value caml_unix_tcgetattr(value fd)
+{ caml_invalid_argument("tcgetattr not implemented"); }
 
-CAMLprim value unix_tcsetattr(value fd, value when, value arg)
-{ invalid_argument("tcsetattr not implemented"); }
+CAMLprim value caml_unix_tcsetattr(value fd, value when, value arg)
+{ caml_invalid_argument("tcsetattr not implemented"); }
 
-CAMLprim value unix_tcsendbreak(value fd, value delay)
-{ invalid_argument("tcsendbreak not implemented"); }
+CAMLprim value caml_unix_tcsendbreak(value fd, value delay)
+{ caml_invalid_argument("tcsendbreak not implemented"); }
 
-CAMLprim value unix_tcdrain(value fd)
-{ invalid_argument("tcdrain not implemented"); }
+CAMLprim value caml_unix_tcdrain(value fd)
+{ caml_invalid_argument("tcdrain not implemented"); }
 
-CAMLprim value unix_tcflush(value fd, value queue)
-{ invalid_argument("tcflush not implemented"); }
+CAMLprim value caml_unix_tcflush(value fd, value queue)
+{ caml_invalid_argument("tcflush not implemented"); }
 
-CAMLprim value unix_tcflow(value fd, value action)
-{ invalid_argument("tcflow not implemented"); }
+CAMLprim value caml_unix_tcflow(value fd, value action)
+{ caml_invalid_argument("tcflow not implemented"); }
 
 #endif
