@@ -25,10 +25,14 @@
 #include "caml/intext.h"
 #include "caml/osdeps.h"
 #include "caml/fail.h"
+#include "caml/frame_descriptors.h"
+#include "caml/globroots.h"
 #include "caml/signals.h"
 
 #include "caml/hooks.h"
 
+intnat caml_globals_inited = 0;
+
 CAMLexport void (*caml_natdynlink_hook)(void* handle, const char* unit) = NULL;
 
 #include <stdio.h>
@@ -51,6 +55,8 @@ static void *getsym(void *handle, const char *module, const char *name){
   return sym;
 }
 
+extern char caml_globals_map[];
+
 CAMLprim value caml_natdynlink_getmap(value unit)
 {
   return caml_input_value_from_block(caml_globals_map, INT_MAX);
@@ -111,16 +117,13 @@ CAMLprim value caml_natdynlink_run(value handle_v, value symbol) {
   sym = optsym("__gc_roots");
   if (NULL != sym) caml_register_dyn_global(sym);
 
-  sym = optsym("__data_begin");
-  sym2 = optsym("__data_end");
-  if (NULL != sym && NULL != sym2)
-    caml_page_table_add(In_static_data, sym, sym2);
-
   sym = optsym("__code_begin");
   sym2 = optsym("__code_end");
-  if (NULL != sym && NULL != sym2)
+  /* Do not register empty code fragments */
+  if (NULL != sym && NULL != sym2 && sym != sym2) {
     caml_register_code_fragment((char *) sym, (char *) sym2,
                                 DIGEST_LATER, NULL);
+  }
 
   if( caml_natdynlink_hook != NULL ) caml_natdynlink_hook(handle,unit);
 
