@@ -1,17 +1,17 @@
-(***********************************************************************)
-(*                                                                     *)
-(*                           Objective Caml                            *)
-(*                                                                     *)
-(*  Xavier Leroy and Pierre Weis, projet Cristal, INRIA Rocquencourt   *)
-(*                                                                     *)
-(*  Copyright 1996 Institut National de Recherche en Informatique et   *)
-(*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License, with    *)
-(*  the special exception on linking described in file ../LICENSE.     *)
-(*                                                                     *)
-(***********************************************************************)
-
-(* $Id$ *)
+(**************************************************************************)
+(*                                                                        *)
+(*                                 OCaml                                  *)
+(*                                                                        *)
+(*   Xavier Leroy and Pierre Weis, projet Cristal, INRIA Rocquencourt     *)
+(*                                                                        *)
+(*   Copyright 1996 Institut National de Recherche en Informatique et     *)
+(*     en Automatique.                                                    *)
+(*                                                                        *)
+(*   All rights reserved.  This file is distributed under the terms of    *)
+(*   the GNU Lesser General Public License version 2.1, with the          *)
+(*   special exception on linking described in the file LICENSE.          *)
+(*                                                                        *)
+(**************************************************************************)
 
 (** Formatted output functions. *)
 
@@ -27,7 +27,7 @@ val fprintf : out_channel -> ('a, out_channel, unit) format -> 'a
 
    Conversion specifications have the following form:
 
-   [% \[flags\] \[width\] \[.precision\] type]
+   [% [flags] [width] [.precision] type]
 
    In short, a conversion specification consists in the [%] character,
    followed by optional modifiers and a type which is made of one or
@@ -36,26 +36,39 @@ val fprintf : out_channel -> ('a, out_channel, unit) format -> 'a
    The types and their meanings are:
 
    - [d], [i]: convert an integer argument to signed decimal.
+     The flag [#] adds underscores to large values for readability.
    - [u], [n], [l], [L], or [N]: convert an integer argument to
      unsigned decimal.  Warning: [n], [l], [L], and [N] are
      used for [scanf], and should not be used for [printf].
+     The flag [#] adds underscores to large values for readability.
    - [x]: convert an integer argument to unsigned hexadecimal,
      using lowercase letters.
+     The flag [#] adds a [0x] prefix to non zero values.
    - [X]: convert an integer argument to unsigned hexadecimal,
      using uppercase letters.
+     The flag [#] adds a [0X] prefix to non zero values.
    - [o]: convert an integer argument to unsigned octal.
+     The flag [#] adds a [0] prefix to non zero values.
    - [s]: insert a string argument.
-   - [S]: convert a string argument to Caml syntax (double quotes, escapes).
+   - [S]: convert a string argument to OCaml syntax (double quotes, escapes).
    - [c]: insert a character argument.
-   - [C]: convert a character argument to Caml syntax (single quotes, escapes).
+   - [C]: convert a character argument to OCaml syntax
+     (single quotes, escapes).
    - [f]: convert a floating-point argument to decimal notation,
      in the style [dddd.ddd].
-   - [F]: convert a floating-point argument to Caml syntax ([dddd.]
+   - [F]: convert a floating-point argument to OCaml syntax ([dddd.]
      or [dddd.ddd] or [d.ddd e+-dd]).
+     Converts to hexadecimal with the [#] flag (see [h]).
    - [e] or [E]: convert a floating-point argument to decimal notation,
      in the style [d.ddd e+-dd] (mantissa and exponent).
    - [g] or [G]: convert a floating-point argument to decimal notation,
-     in style [f] or [e], [E] (whichever is more compact).
+     in style [f] or [e], [E] (whichever is more compact). Moreover,
+     any trailing zeros are removed from the fractional part of the result
+     and the decimal-point character is removed if there is no fractional
+     part remaining.
+   - [h] or [H]: convert a floating-point argument to hexadecimal notation,
+     in the style [0xh.hhhh p+-dd] (hexadecimal mantissa, exponent in
+     decimal and denotes a power of 2).
    - [B]: convert a boolean argument to the string [true] or [false]
    - [b]: convert a boolean argument (deprecated; do not use in new
      programs).
@@ -73,8 +86,9 @@ val fprintf : out_channel -> ('a, out_channel, unit) format -> 'a
      [fprintf] at the current point.
    - [t]: same as [%a], but take only one argument (with type
      [out_channel -> unit]) and apply it to [outchan].
-   - [\{ fmt %\}]: convert a format string argument. The argument must
-     have the same type as the internal format string [fmt].
+   - [\{ fmt %\}]: convert a format string argument to its type digest.
+     The argument must have the same type as the internal format string
+     [fmt].
    - [( fmt %)]: format string substitution. Take a format string
      argument and substitute it to the internal format string [fmt]
      to print following arguments. The argument must have the same
@@ -82,7 +96,8 @@ val fprintf : out_channel -> ('a, out_channel, unit) format -> 'a
    - [!]: take no argument and flush the output.
    - [%]: take no argument and output one [%] character.
    - [\@]: take no argument and output one [\@] character.
-   - [,]: take no argument and do nothing.
+   - [,]: take no argument and output nothing: a no-op delimiter for
+     conversion specifications.
 
    The optional [flags] are:
    - [-]: left-justify the output (default is right justification).
@@ -91,7 +106,8 @@ val fprintf : out_channel -> ('a, out_channel, unit) format -> 'a
      sign if positive.
    - space: for signed numerical conversions, prefix number with a
      space if positive.
-   - [#]: request an alternate formatting style for numbers.
+   - [#]: request an alternate formatting style for the integer types
+     and the floating-point type [F].
 
    The optional [width] is an integer indicating the minimal
    width of the result. For instance, [%6d] prints an integer,
@@ -99,8 +115,9 @@ val fprintf : out_channel -> ('a, out_channel, unit) format -> 'a
 
    The optional [precision] is a dot [.] followed by an integer
    indicating how many digits follow the decimal point in the [%f],
-   [%e], and [%E] conversions. For instance, [%.4f] prints a [float] with
-   4 fractional digits.
+   [%e], [%E], [%h], and [%H] conversions or the maximum number of
+   significant digits to appear for the [%F], [%g] and [%G] conversions.
+   For instance, [%.4f] prints a [float] with 4 fractional digits.
 
    The integer in a [width] or [precision] can also be specified as
    [*], in which case an extra integer argument is taken to specify
@@ -115,12 +132,6 @@ val printf : ('a, out_channel, unit) format -> 'a
 val eprintf : ('a, out_channel, unit) format -> 'a
 (** Same as {!Printf.fprintf}, but output on [stderr]. *)
 
-val ifprintf : 'a -> ('b, 'a, unit) format -> 'b
-(** Same as {!Printf.fprintf}, but does not print anything.
-    Useful to ignore some material when conditionally printing.
-    @since 3.10.0
-*)
-
 val sprintf : ('a, unit, string) format -> 'a
 (** Same as {!Printf.fprintf}, but instead of printing on an output channel,
    return a string containing the result of formatting the arguments. *)
@@ -130,97 +141,55 @@ val bprintf : Buffer.t -> ('a, Buffer.t, unit) format -> 'a
    append the formatted arguments to the given extensible buffer
    (see module {!Buffer}). *)
 
+val ifprintf : 'b -> ('a, 'b, 'c, unit) format4 -> 'a
+(** Same as {!Printf.fprintf}, but does not print anything.
+    Useful to ignore some material when conditionally printing.
+    @since 3.10.0
+*)
+
+val ibprintf : Buffer.t -> ('a, Buffer.t, unit) format -> 'a
+(** Same as {!Printf.bprintf}, but does not print anything.
+    Useful to ignore some material when conditionally printing.
+    @since 4.11.0
+*)
+
 (** Formatted output functions with continuations. *)
 
-val kfprintf : (out_channel -> 'a) -> out_channel ->
-              ('b, out_channel, unit, 'a) format4 -> 'b;;
+val kfprintf : (out_channel -> 'd) -> out_channel ->
+              ('a, out_channel, unit, 'd) format4 -> 'a
 (** Same as [fprintf], but instead of returning immediately,
    passes the out channel to its first argument at the end of printing.
    @since 3.09.0
 *)
 
-val ksprintf : (string -> 'a) -> ('b, unit, string, 'a) format4 -> 'b;;
+val ikfprintf : ('b -> 'd) -> 'b -> ('a, 'b, 'c, 'd) format4 -> 'a
+(** Same as [kfprintf] above, but does not print anything.
+   Useful to ignore some material when conditionally printing.
+   @since 4.01.0
+*)
+
+val ksprintf : (string -> 'd) -> ('a, unit, string, 'd) format4 -> 'a
 (** Same as [sprintf] above, but instead of returning the string,
    passes it to the first argument.
    @since 3.09.0
 *)
 
-val kbprintf : (Buffer.t -> 'a) -> Buffer.t ->
-              ('b, Buffer.t, unit, 'a) format4 -> 'b;;
+val kbprintf : (Buffer.t -> 'd) -> Buffer.t ->
+              ('a, Buffer.t, unit, 'd) format4 -> 'a
 (** Same as [bprintf], but instead of returning immediately,
    passes the buffer to its first argument at the end of printing.
    @since 3.10.0
 *)
 
+val ikbprintf : (Buffer.t -> 'd) -> Buffer.t ->
+               ('a, Buffer.t, unit, 'd) format4 -> 'a
+(** Same as [kbprintf] above, but does not print anything.
+   Useful to ignore some material when conditionally printing.
+   @since 4.11.0
+*)
+
 (** Deprecated *)
 
-val kprintf : (string -> 'a) -> ('b, unit, string, 'a) format4 -> 'b;;
+val kprintf : (string -> 'b) -> ('a, unit, string, 'b) format4 -> 'a
+[@@ocaml.deprecated "Use Printf.ksprintf instead."]
 (** A deprecated synonym for [ksprintf]. *)
-
-(**/**)
-
-(* For Caml system internal use only. Don't call directly. *)
-
-module CamlinternalPr : sig
-
-  module Sformat : sig
-    type index;;
-
-    val index_of_int : int -> index;;
-    external int_of_index : index -> int = "%identity";;
-    external unsafe_index_of_int : int -> index = "%identity";;
-
-    val succ_index : index -> index;;
-
-    val sub : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> index -> int -> string;;
-    val to_string : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> string;;
-    external length : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> int
-      = "%string_length";;
-    external get : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> int -> char
-      = "%string_safe_get";;
-    external unsafe_to_string : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> string
-      = "%identity";;
-    external unsafe_get : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> int -> char
-      = "%string_unsafe_get";;
-
-  end;;
-
-  module Tformat : sig
-
-    type ac = {
-      mutable ac_rglr : int;
-      mutable ac_skip : int;
-      mutable ac_rdrs : int;
-    };;
-
-    val ac_of_format : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> ac;;
-
-    val sub_format :
-        (('a, 'b, 'c, 'd, 'e, 'f) format6 -> int) ->
-        (('a, 'b, 'c, 'd, 'e, 'f) format6 -> int -> char -> int) ->
-        char ->
-        ('a, 'b, 'c, 'd, 'e, 'f) format6 ->
-        int ->
-        int
-
-    val summarize_format_type : ('a, 'b, 'c, 'd, 'e, 'f) format6 -> string
-
-    val scan_format : ('a, 'b, 'c, 'd, 'e, 'f) format6 ->
-        'g array ->
-        Sformat.index ->
-        int ->
-        (Sformat.index -> string -> int -> 'h) ->
-        (Sformat.index -> 'i -> 'j -> int -> 'h) ->
-        (Sformat.index -> 'k -> int -> 'h) ->
-        (Sformat.index -> int -> 'h) ->
-        (Sformat.index -> ('l, 'm, 'n, 'o, 'p, 'q) format6 -> int -> 'h) ->
-        'h
-
-    val kapr :
-        (('a, 'b, 'c, 'd, 'e, 'f) format6 -> Obj.t array -> 'g) ->
-        ('a, 'b, 'c, 'd, 'e, 'f) format6 ->
-        'g
-
-  end;;
-
-end;;
