@@ -1,14 +1,18 @@
-#define CAML_NAME_SPACE
 #define CAML_INTERNALS
 
+#include <caml/mlvalues.h>
 #include <caml/domain_state.h>
 #include <caml/signals.h>
 
 CAMLprim value request_minor_gc(value v) {
   Caml_state->requested_minor_gc = 1;
   Caml_state->requested_major_slice = 1;
-  caml_something_to_do = 1;
-  Caml_state->young_limit = Caml_state->young_alloc_end;
+  /*
+    This is massively unsafe in multicore but the polling
+    tests are only run in a single domain, so we're probably
+    good.
+  */
+  Caml_state->young_limit = (uintnat)Caml_state->young_end;
 
   return Val_unit;
 }
