@@ -21,8 +21,6 @@ PREFIX=~/local
 MAKE="make $MAKE_ARG"
 SHELL=dash
 
-MAKE_WARN="$MAKE --warn-undefined-variables"
-
 export PATH=$PREFIX/bin:$PATH
 
 Configure () {
@@ -31,7 +29,6 @@ Configure () {
 ------------------------------------------------------------------------
 This test builds the OCaml compiler distribution with your pull request
 and runs its testsuite.
-
 Failing to build the compiler distribution, or testsuite failures are
 critical errors that must be understood and fixed before your pull
 request can be merged.
@@ -40,9 +37,7 @@ EOF
 
   configure_flags="\
     --prefix=$PREFIX \
-    --enable-flambda-invariants \
-    --enable-ocamltest \
-    --disable-dependency-generation \
+    --enable-debug-runtime \
     $CONFIG_ARG"
 
   case $XARCH in
@@ -51,7 +46,9 @@ EOF
     ;;
   i386)
     ./configure --build=x86_64-pc-linux-gnu --host=i386-linux \
-      CC='gcc -m32' AS='as --32' ASPP='gcc -m32 -c' \
+      CC='gcc -m32 -march=x86-64' \
+      AS='as --32' \
+      ASPP='gcc -m32 -march=x86-64 -c' \
       PARTIALLD='ld -r -melf_i386' \
       $configure_flags
     ;;
@@ -63,18 +60,23 @@ EOF
 }
 
 Build () {
-  script --return --command "$MAKE_WARN world.opt" build.log
-  script --return --append --command "$MAKE_WARN ocamlnat" build.log
+  $MAKE world.opt
   echo Ensuring that all names are prefixed in the runtime
-  ./tools/check-symbol-names runtime/*.a
+  ./tools/check-symbol-names runtime/*.a otherlibs/*/lib*.a
 }
 
 Test () {
-  cd testsuite
-  echo Running the testsuite with the normal runtime
-  $MAKE all
-  echo Running the testsuite with the debug runtime
-  $MAKE USE_RUNTIME='d' OCAMLTESTDIR="$(pwd)/_ocamltestd" TESTLOG=_logd all
+  echo Running the testsuite
+  $MAKE -C testsuite parallel
+  cd ..
+}
+
+# By default, TestPrefix will attempt to run the tests
+# in the given directory in parallel.
+TestPrefix () {
+  TO_RUN=parallel-"$1"
+  echo Running single testsuite directory with $TO_RUN
+  $MAKE -C testsuite $TO_RUN
   cd ..
 }
 
@@ -88,15 +90,6 @@ Install () {
 }
 
 Checks () {
-  set +x
-  STATUS=0
-  if grep -Fq ' warning: undefined variable ' build.log; then
-    echo -e '\e[31mERROR\e[0m Undefined Makefile variables detected!'
-    grep -F ' warning: undefined variable ' build.log | sort | uniq
-    STATUS=1
-  fi
-  rm build.log
-  set -x
   if fgrep 'SUPPORTS_SHARED_LIBRARIES=true' Makefile.config &>/dev/null ; then
     echo Check the code examples in the manual
     $MAKE manual-pregen
@@ -118,7 +111,6 @@ Checks () {
   test -z "$(git status --porcelain)"
   # Check that there are no ignored files
   test -z "$(git ls-files --others -i --exclude-standard)"
-  exit $STATUS
 }
 
 CheckManual () {
@@ -173,6 +165,7 @@ case $1 in
 configure) Configure;;
 build) Build;;
 test) Test;;
+test_prefix) TestPrefix $2;;
 api-docs) API_Docs;;
 install) Install;;
 manual) BuildManual;;
