@@ -53,41 +53,47 @@ function set_configuration {
     case "$1" in
         cygwin*)
             dep='--disable-dependency-generation'
+            man=''
         ;;
         mingw32)
             build='--build=i686-pc-cygwin'
             host='--host=i686-w64-mingw32'
             dep='--disable-dependency-generation'
+            man=''
         ;;
         mingw64)
             build='--build=i686-pc-cygwin'
             host='--host=x86_64-w64-mingw32'
             dep='--disable-dependency-generation'
+            man='--disable-stdlib-manpages'
         ;;
         msvc32)
             build='--build=i686-pc-cygwin'
             host='--host=i686-pc-windows'
             dep='--disable-dependency-generation'
+            man=''
         ;;
         msvc64)
             build='--build=x86_64-pc-cygwin'
             host='--host=x86_64-pc-windows'
             # Explicitly test dependency generation on msvc64
             dep='--enable-dependency-generation'
+            man=''
         ;;
     esac
 
     mkdir -p "$CACHE_DIRECTORY"
     ./configure --cache-file="$CACHE_DIRECTORY/config.cache-$1" \
-                $dep $build $host --prefix="$2" --enable-ocamltest || ( \
+                $dep $build $man $host --prefix="$2" --enable-ocamltest || ( \
       rm -f "$CACHE_DIRECTORY/config.cache-$1" ; \
       ./configure --cache-file="$CACHE_DIRECTORY/config.cache-$1" \
-                  $dep $build $host --prefix="$2" --enable-ocamltest )
+                  $dep $build $man $host --prefix="$2" --enable-ocamltest )
 
 #    FILE=$(pwd | cygpath -f - -m)/Makefile.config
 #    run "Content of $FILE" cat Makefile.config
 }
 
+PARALLEL_URL='https://git.savannah.gnu.org/cgit/parallel.git/plain/src/parallel'
 APPVEYOR_BUILD_FOLDER=$(echo "$APPVEYOR_BUILD_FOLDER" | cygpath -f -)
 FLEXDLLROOT="$PROGRAMFILES/flexdll"
 OCAMLROOT=$(echo "$OCAMLROOT" | cygpath -f - -m)
@@ -101,6 +107,15 @@ fi
 
 case "$1" in
   install)
+    if [ ! -e "$CACHE_DIRECTORY/parallel-source" ] || \
+       [ "$PARALLEL_URL" != "$(cat "$CACHE_DIRECTORY/parallel-source")" ] ; then
+      # Download latest version directly from the repo
+      curl -Ls $PARALLEL_URL -o "$CACHE_DIRECTORY/parallel"
+      echo "$PARALLEL_URL" > "$CACHE_DIRECTORY/parallel-source"
+    fi
+    cp "$CACHE_DIRECTORY/parallel" /usr/bin
+    chmod +x /usr/bin/parallel
+    parallel --version
     if [[ $BOOTSTRAP_FLEXDLL = 'false' ]] ; then
       mkdir -p "$FLEXDLLROOT"
       cd "$APPVEYOR_BUILD_FOLDER/../flexdll"
@@ -119,17 +134,23 @@ case "$1" in
   test)
     FULL_BUILD_PREFIX="$APPVEYOR_BUILD_FOLDER/../$BUILD_PREFIX"
     run 'ocamlc.opt -version' "$FULL_BUILD_PREFIX-$PORT/ocamlc.opt" -version
-    if [[ $PORT = 'mingw32' ]] ; then
+    if [[ $PORT =~ mingw* ]] ; then
       run "Check runtime symbols" \
           "$FULL_BUILD_PREFIX-$PORT/tools/check-symbol-names" \
-          $FULL_BUILD_PREFIX-$PORT/runtime/*.a
+          $FULL_BUILD_PREFIX-$PORT/runtime/*.a \
+          $FULL_BUILD_PREFIX-$PORT/otherlibs/*/lib*.a
     fi
-    if [[ $PORT = 'mingw64' ]] ; then
-      export PATH="$PATH:/usr/x86_64-w64-mingw32/sys-root/mingw/bin"
-    elif [[ $PORT = 'mingw32' ]] ; then
-      export PATH="$PATH:/usr/i686-w64-mingw32/sys-root/mingw/bin"
-    fi
-    run "test $PORT" $MAKE -C "$FULL_BUILD_PREFIX-$PORT" tests
+    # The testsuite is too slow to run on AppVeyor in full. Run the dynlink
+    # tests now (to include natdynlink)
+    run "test dynlink $PORT" \
+        $MAKE -C "$FULL_BUILD_PREFIX-$PORT/testsuite" parallel-lib-dynlink
+    # Now reconfigure ocamltest to run in bytecode-only mode
+    sed -i '/native_/s/true/false/' \
+           "$FULL_BUILD_PREFIX-$PORT/ocamltest/ocamltest_config.ml"
+    $MAKE -C "$FULL_BUILD_PREFIX-$PORT/ocamltest" -j all allopt
+    # And run the entire testsuite, skipping all the native-code tests
+    run "test $PORT" \
+        make -C "$FULL_BUILD_PREFIX-$PORT/testsuite" SHOW_TIMINGS=1 all
     run "install $PORT" $MAKE -C "$FULL_BUILD_PREFIX-$PORT" install
     if [[ $PORT = 'msvc64' ]] ; then
       run "$MAKE check_all_arches" \
