@@ -20,8 +20,8 @@ open Location
 open Parsetree
 
 (* User programs must not use identifiers that start with these prefixes. *)
-let idprefix = "__ocaml_prof_";;
-let modprefix = "OCAML__prof_";;
+let idprefix = "__ocaml_prof_"
+let modprefix = "OCAML__prof_"
 
 (* Errors specific to the profiler *)
 exception Profiler of string
@@ -64,19 +64,17 @@ let copy next =
   assert (next >= !cur_point);
   seek_in !inchan !cur_point;
   copy_chars (next - !cur_point);
-  cur_point := next;
-;;
+  cur_point := next
 
-let prof_counter = ref 0;;
+let prof_counter = ref 0
 
 let instr_mode = ref false
 
-type insert = Open | Close;;
-let to_insert = ref ([] : (insert * int) list);;
+type insert = Open | Close
+let to_insert = ref ([] : (insert * int) list)
 
 let insert_action st en =
   to_insert := (Open, st) :: (Close, en) :: !to_insert
-;;
 
 (* Producing instrumented code *)
 let add_incr_counter modul (kind,pos) =
@@ -85,9 +83,8 @@ let add_incr_counter modul (kind,pos) =
    | Open ->
          fprintf !outchan "(%sProfiling.incr %s%s_cnt %d; "
                  modprefix idprefix modul !prof_counter;
-         incr prof_counter;
-   | Close -> fprintf !outchan ")";
-;;
+         incr prof_counter
+   | Close -> fprintf !outchan ")"
 
 let counters = ref (Array.make 0 0)
 
@@ -101,7 +98,6 @@ let add_val_counter (kind,pos) =
     fprintf !outchan "(* %s%d *) " !special_id !counters.(!prof_counter);
     incr prof_counter;
   end
-;;
 
 (* ************* rewrite ************* *)
 
@@ -116,7 +112,6 @@ let insert_profile rw_exp ex =
     insert_action st en;
     rw_exp false ex;
   end
-;;
 
 
 let pos_len = ref 0
@@ -143,12 +138,11 @@ let final_rewrite add_function =
     if String.length len > 9 then raise (Profiler "too many counters");
     seek_out !outchan (!pos_len - String.length len);
     output_string !outchan len
-  end;
+  end
   (* Cannot close because outchan is stdout and Format doesn't like
      a closed stdout.
     close_out !outchan;
   *)
-;;
 
 let rec rewrite_patexp_list iflag l =
   rewrite_exp_list iflag (List.map (fun x -> x.pvb_expr) l)
@@ -416,7 +410,6 @@ let null_rewrite srcfile =
   inchan := open_in_bin srcfile;
   copy (in_channel_length !inchan);
   close_in !inchan
-;;
 
 (* Setting flags from saved config *)
 let set_flags s =
@@ -440,7 +433,7 @@ let dumpfile = ref "ocamlprof.dump"
 
 (* Process a file *)
 
-let process_intf_file filename = null_rewrite filename;;
+let process_intf_file filename = null_rewrite filename
 
 let process_impl_file filename =
    let modname = Filename.basename(Filename.chop_extension filename) in
@@ -467,14 +460,12 @@ let process_impl_file filename =
      init_rewrite modes modname;
      rewrite_file filename add_val_counter;
    end
-;;
 
 let process_anon_file filename =
   if Filename.check_suffix filename ".ml" then
     process_impl_file filename
   else
     process_intf_file filename
-;;
 
 (* Main function *)
 
@@ -484,13 +475,11 @@ let usage = "Usage: ocamlprof <options> <files>\noptions are:"
 
 let print_version () =
   printf "ocamlprof, version %s@." Sys.ocaml_version;
-  exit 0;
-;;
+  exit 0
 
 let print_version_num () =
   printf "%s@." Sys.ocaml_version;
-  exit 0;
-;;
+  exit 0
 
 let main () =
   try
