@@ -1,19 +1,20 @@
-/***********************************************************************/
-/*                                                                     */
-/*                           Objective Caml                            */
-/*                                                                     */
-/*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         */
-/*                                                                     */
-/*  Copyright 1996 Institut National de Recherche en Informatique et   */
-/*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the Q Public License version 1.0.               */
-/*                                                                     */
-/***********************************************************************/
+/**************************************************************************/
+/*                                                                        */
+/*                                 OCaml                                  */
+/*                                                                        */
+/*             Xavier Leroy, projet Cristal, INRIA Rocquencourt           */
+/*                                                                        */
+/*   Copyright 1996 Institut National de Recherche en Informatique et     */
+/*     en Automatique.                                                    */
+/*                                                                        */
+/*   All rights reserved.  This file is distributed under the terms of    */
+/*   the GNU Lesser General Public License version 2.1, with the          */
+/*   special exception on linking described in the file LICENSE.          */
+/*                                                                        */
+/**************************************************************************/
 
 /* Based on public-domain code from Berkeley Yacc */
 
-/* $Id$ */
-
 #include "defs.h"
 
 short *itemset;
@@ -25,9 +26,9 @@ static unsigned *EFF;
 
 
 
-void print_EFF ();
-void print_first_derives ();
-void print_closure ();
+void print_EFF (void);
+void print_first_derives (void);
+void print_closure (void);
 
 void set_EFF(void)
 {
@@ -163,7 +164,7 @@ void closure(short int *nucleus, int n)
     {
         word = *rsp;
         if (word == 0)
-            ruleno += BITS_PER_WORD;
+            ruleno += BITS_PER_INT;
         else
         {
             mask = 1;
