@@ -1,18 +1,23 @@
-/***********************************************************************/
-/*                                                                     */
-/*                           Objective Caml                            */
-/*                                                                     */
-/*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         */
-/*                                                                     */
-/*  Copyright 1996 Institut National de Recherche en Informatique et   */
-/*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the Q Public License version 1.0.               */
-/*                                                                     */
-/***********************************************************************/
+/**************************************************************************/
+/*                                                                        */
+/*                                 OCaml                                  */
+/*                                                                        */
+/*             Xavier Leroy, projet Cristal, INRIA Rocquencourt           */
+/*                                                                        */
+/*   Copyright 1996 Institut National de Recherche en Informatique et     */
+/*     en Automatique.                                                    */
+/*                                                                        */
+/*   All rights reserved.  This file is distributed under the terms of    */
+/*   the GNU Lesser General Public License version 2.1, with the          */
+/*   special exception on linking described in the file LICENSE.          */
+/*                                                                        */
+/**************************************************************************/
 
 /* Based on public-domain code from Berkeley Yacc */
 
-/* $Id$ */
+#ifndef DEBUG
+#define NDEBUG
+#endif
 
 #include <assert.h>
 #include <ctype.h>
@@ -20,7 +25,14 @@
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include "../config/s.h"
+#include <string.h>
+#define CAML_INTERNALS
+#include "caml/config.h"
+#include "caml/mlvalues.h"
+#include "caml/osdeps.h"
+#include "caml/misc.h"
+
+#define caml_stat_strdup strdup
 
 /*  machine-dependent definitions                              */
 /*  the following definitions are for the Tahoe                */
@@ -30,7 +42,7 @@
 /*  MAXSHORT is the largest value of a C short                 */
 /*  MINSHORT is the most negative value of a C short           */
 /*  MAXTABLE is the maximum table size                         */
-/*  BITS_PER_WORD is the number of bits in a C unsigned        */
+/*  BITS_PER_INT is the number of bits in a C unsigned         */
 /*  WORDSIZE computes the number of words needed to            */
 /*        store n bits                                         */
 /*  BIT returns the value of the n-th bit starting             */
@@ -42,10 +54,10 @@
 #define MINSHORT        SHRT_MIN
 #define MAXTABLE        32500
 
-#define BITS_PER_WORD        (8*sizeof(unsigned))
-#define        WORDSIZE(n)        (((n)+(BITS_PER_WORD-1))/BITS_PER_WORD)
-#define        BIT(r, n)        ((((r)[(n)/BITS_PER_WORD])>>((n)%BITS_PER_WORD))&1)
-#define        SETBIT(r, n)        ((r)[(n)/BITS_PER_WORD]|=(1<<((n)%BITS_PER_WORD)))
+#define BITS_PER_INT    (8*sizeof(unsigned))
+#define WORDSIZE(n)     (((n)+(BITS_PER_INT-1))/BITS_PER_INT)
+#define BIT(r, n)       ((((r)[(n)/BITS_PER_INT])>>((n)%BITS_PER_INT))&1)
+#define SETBIT(r, n)    ((r)[(n)/BITS_PER_INT]|=(1<<((n)%BITS_PER_INT)))
 
 /*  character names  */
 
@@ -64,11 +76,9 @@
 
 /* defines for constructing filenames */
 
-#define CODE_SUFFIX        ".code.c"
-#define        DEFINES_SUFFIX        ".tab.h"
-#define        OUTPUT_SUFFIX        ".ml"
-#define        VERBOSE_SUFFIX        ".output"
-#define INTERFACE_SUFFIX ".mli"
+#define        OUTPUT_SUFFIX        T(".ml")
+#define        VERBOSE_SUFFIX        T(".output")
+#define INTERFACE_SUFFIX T(".mli")
 
 /* keyword codes */
 
@@ -80,8 +90,6 @@
 #define TEXT 5
 #define TYPE 6
 #define START 7
-#define UNION 8
-#define IDENT 9
 
 /*  symbol classes  */
 
@@ -138,10 +146,16 @@ struct bucket
     short prec;
     char class;
     char assoc;
-    char entry;
+    unsigned char entry;  /* 1..MAX_ENTRY_POINT (0 for unassigned) */
     char true_token;
 };
 
+/* MAX_ENTRY_POINT is the maximal number of entry points into the grammar. */
+/* Entry points are identified by a non-zero byte in the input stream,     */
+/* so there are at most 255 entry points.                                  */
+
+#define MAX_ENTRY_POINT MAXCHAR
+
 /* TABLE_SIZE is the number of entries in the symbol table.      */
 /* TABLE_SIZE must be a power of two.                            */
 
@@ -202,41 +216,43 @@ struct action
 
 /* global variables */
 
-extern char dflag;
 extern char lflag;
 extern char rflag;
 extern char tflag;
 extern char vflag;
 extern char qflag;
 extern char sflag;
+extern char eflag;
 extern char big_endian;
 
+/* myname should be UTF-8 encoded */
 extern char *myname;
 extern char *cptr;
 extern char *line;
 extern int lineno;
+/* virtual_input_file_name should be UTF-8 encoded */
 extern char *virtual_input_file_name;
 extern int outline;
 
-extern char *action_file_name;
-extern char *entry_file_name;
-extern char *code_file_name;
-extern char *defines_file_name;
-extern char *input_file_name;
-extern char *output_file_name;
-extern char *text_file_name;
-extern char *union_file_name;
-extern char *verbose_file_name;
-extern char *interface_file_name;
+extern char_os *action_file_name;
+extern char_os *entry_file_name;
+extern char_os *code_file_name;
+extern char_os *input_file_name;
+extern char_os *output_file_name;
+extern char_os *text_file_name;
+extern char_os *verbose_file_name;
+extern char_os *interface_file_name;
+
+/* UTF-8 versions of code_file_name and input_file_name */
+extern char *code_file_name_disp;
+extern char *input_file_name_disp;
 
 extern FILE *action_file;
 extern FILE *entry_file;
 extern FILE *code_file;
-extern FILE *defines_file;
 extern FILE *input_file;
 extern FILE *output_file;
 extern FILE *text_file;
-extern FILE *union_file;
 extern FILE *verbose_file;
 extern FILE *interface_file;
 
@@ -248,8 +264,7 @@ extern int ntokens;
 extern int nvars;
 extern int ntags;
 
-extern char unionized;
-extern char line_format[];
+#define line_format "# %d \"%s\"\n"
 
 extern int   start_symbol;
 extern char  **symbol_name;
@@ -298,13 +313,6 @@ extern short final_state;
 
 /* global functions */
 
-#ifdef __GNUC__
-/* Works only in GCC 2.5 and later */
-#define Noreturn __attribute ((noreturn))
-#else
-#define Noreturn
-#endif
-
 extern char *allocate(unsigned int n);
 extern bucket *lookup(char *name);
 extern bucket *make_bucket(char *name);
@@ -314,45 +322,44 @@ extern action *add_reductions(int stateno, register action *actions);
 extern action *add_reduce(register action *actions, register int ruleno, register int symbol);
 extern void closure (short int *nucleus, int n);
 extern void create_symbol_table (void);
-extern void default_action_error (void);
+extern void default_action_error (void) Noreturn;
 extern void done (int k) Noreturn;
-extern void entry_without_type (char *s);
-extern void fatal (char *msg);
+extern void entry_without_type (char *s) Noreturn;
+extern void fatal (char *msg) Noreturn;
 extern void finalize_closure (void);
 extern void free_parser (void);
 extern void free_symbol_table (void);
 extern void free_symbols (void);
-extern void illegal_character (char *c_cptr);
-extern void illegal_token_ref (int i, char *name);
+extern void illegal_character (char *c_cptr) Noreturn;
+extern void illegal_token_ref (int i, char *name) Noreturn;
 extern void lalr (void);
 extern void lr0 (void);
 extern void make_parser (void);
-extern void no_grammar (void);
-extern void no_space (void);
-extern void open_error (char *filename);
+extern void no_grammar (void) Noreturn;
+extern void no_space (void) Noreturn;
+extern void open_error (char_os *filename) Noreturn;
 extern void output (void);
-extern void over_unionized (char *u_cptr);
 extern void prec_redeclared (void);
-extern void polymorphic_entry_point(char *s);
+extern void polymorphic_entry_point(char *s) Noreturn;
+extern void forbidden_conflicts (void);
 extern void reader (void);
 extern void reflexive_transitive_closure (unsigned int *R, int n);
 extern void reprec_warning (char *s);
 extern void retyped_warning (char *s);
 extern void revalued_warning (char *s);
 extern void set_first_derives (void);
-extern void syntax_error (int st_lineno, char *st_line, char *st_cptr) Noreturn, terminal_lhs (int s_lineno);
-extern void terminal_start (char *s);
-extern void tokenized_start (char *s);
-extern void too_many_entries (void);
+extern void syntax_error (int st_lineno, char *st_line, char *st_cptr) Noreturn, terminal_lhs (int s_lineno) Noreturn;
+extern void terminal_start (char *s) Noreturn;
+extern void tokenized_start (char *s) Noreturn;
+extern void too_many_entries (void) Noreturn;
 extern void undefined_goal (char *s);
 extern void undefined_symbol (char *s);
-extern void unexpected_EOF (void);
-extern void unknown_rhs (int i);
-extern void unterminated_action (int a_lineno, char *a_line, char *a_cptr);
-extern void unterminated_comment (int c_lineno, char *c_line, char *c_cptr);
-extern void unterminated_string (int s_lineno, char *s_line, char *s_cptr);
-extern void unterminated_text (int t_lineno, char *t_line, char *t_cptr);
-extern void unterminated_union (int u_lineno, char *u_line, char *u_cptr);
-extern void used_reserved (char *s);
+extern void unexpected_EOF (void) Noreturn;
+extern void unknown_rhs (int i) Noreturn;
+extern void unterminated_action (int a_lineno, char *a_line, char *a_cptr) Noreturn;
+extern void unterminated_comment (int c_lineno, char *c_line, char *c_cptr) Noreturn;
+extern void unterminated_string (int s_lineno, char *s_line, char *s_cptr) Noreturn;
+extern void unterminated_text (int t_lineno, char *t_line, char *t_cptr) Noreturn;
+extern void used_reserved (char *s) Noreturn;
 extern void verbose (void);
 extern void write_section (char **section);
